package org.openxma.dsl.core.generator;

import static java.util.Collections.singletonMap;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.emf.codegen.ecore.genmodel.GenModel;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.xmi.XMLResource;
import org.eclipse.emf.mwe.core.ConfigurationException;
import org.eclipse.emf.mwe.utils.GenModelHelper;
import org.eclipse.xpand2.XpandExecutionContext;
import org.eclipse.xtext.Grammar;

import com.google.common.collect.Sets;

public class EcoreGeneratorFragment extends org.eclipse.xtext.generator.ecore.EcoreGeneratorFragment {

	
	protected GenModel getSaveAndReconcileGenModel(ResourceSet rs, Grammar grammar, XpandExecutionContext ctx,
            List<EPackage> packs) throws ConfigurationException {
        GenModel genModel = getGenModel(rs, grammar, ctx, packs);
        genModel.initialize(packs);
        for (GenPackage genPackage : genModel.getGenPackages()) {
            genPackage.setBasePackage(getBasePackage(grammar));
            genPackage.setClassPackageSuffix("model.impl");
            genPackage.setInterfacePackageSuffix("model");
            if (getFileExtensions() != null && packs.contains(genPackage.getEcorePackage())) {
                genPackage.setFileExtensions(getFileExtensions());
            }
        }
        Set<EPackage> referencedEPackages = getReferencedEPackages(packs);
        List<GenPackage> usedGenPackages = getGenPackagesForPackages(genModel, referencedEPackages);
        reconcileMissingGenPackagesInUsedModels(usedGenPackages);
        genModel.getUsedGenPackages().addAll(usedGenPackages);
        try {
            saveResource(genModel.eResource());
        } catch (IOException e) {
            throw new WrappedException(e);
        }
        new GenModelHelper().registerGenModel(genModel);
        return genModel;
    }
	
	private void reconcileMissingGenPackagesInUsedModels(List<GenPackage> usedGenPackages) {
        Set<GenModel> processedModels = Sets.newHashSet();
        for(GenPackage usedGenPackage: usedGenPackages) {
            GenModel genModel = usedGenPackage.getGenModel();
            if (processedModels.add(genModel)) {
                List<EPackage> missingPackages = genModel.getMissingPackages();
                List<GenPackage> missingGenPackages = getGenPackagesForPackages(genModel, missingPackages);
                genModel.getUsedGenPackages().addAll(missingGenPackages);
            }
        }
    }
	
	private void saveResource(Resource resource) throws IOException {
        Map<String, ToPlatformResourceDeresolvingURIHandler> saveOptions = singletonMap(
                XMLResource.OPTION_URI_HANDLER, new ToPlatformResourceDeresolvingURIHandler());
        resource.save(saveOptions);
    }
    

}
